/*
    Copyright (C) 2019  e Foundation

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package foundation.e.apps.updates

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.os.Build
import android.support.v4.app.NotificationCompat
import android.support.v4.app.NotificationManagerCompat
import foundation.e.apps.MainActivity
import foundation.e.apps.R
import foundation.e.apps.utils.Constants

class UpdatesNotifier {
    private fun getNotification(context: Context,
                                numberOfApps: Int,
                                installAutomatically: Boolean,
                                unmeteredNetworkOnly: Boolean,
                                isConnectedToUnmeteredNetwork: Boolean):
            Notification {
        val notificationBuilder =
                NotificationCompat.Builder(context, Constants.UPDATES_NOTIFICATION_CHANNEL_ID)
        notificationBuilder.setSmallIcon(R.drawable.ic_app_updated_on)
                .priority = NotificationCompat.PRIORITY_DEFAULT
        if (numberOfApps == 1) {
            notificationBuilder.setContentTitle(context.resources.getQuantityString(
                    R.plurals.updates_notification_title,
                    1,
                    numberOfApps))
        } else {
            notificationBuilder.setContentTitle(context.resources.getQuantityString(
                    R.plurals.updates_notification_title,
                    numberOfApps,
                    numberOfApps))
        }
        if (installAutomatically) {
            notificationBuilder.setContentText(context.getString(R.string.updates_notification_text,
                    Constants.AUTOMATICALLY_INSTALL_UPDATES))
            if (unmeteredNetworkOnly && !isConnectedToUnmeteredNetwork) {
                notificationBuilder.setSubText(context
                        .getString(R.string.updates_notification_unmetered_network_warning))
            }
        } else {
            notificationBuilder.setContentText(context.getString(R.string.updates_notification_text,
                    Constants.MANUALLY_INSTALL_UPDATES))
        }
        notificationBuilder.setContentIntent(getClickIntent(context))
        notificationBuilder.setAutoCancel(true)
        return notificationBuilder.build()
    }

    private fun getClickIntent(context: Context): PendingIntent {
        val intent = Intent(context, MainActivity::class.java).apply {
            flags = Intent.FLAG_ACTIVITY_CLEAR_TASK
            putExtra(Constants.UPDATES_NOTIFICATION_CLICK_EXTRA, true)
        }
        return PendingIntent.getActivity(context, 0, intent, 0)
    }

    private fun createNotificationChannel(context: Context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val importance = NotificationManager.IMPORTANCE_DEFAULT
            val channel = NotificationChannel(
                    Constants.UPDATES_NOTIFICATION_CHANNEL_ID,
                    Constants.UPDATES_NOTIFICATION_CHANNEL_TITLE,
                    importance)
            val notificationManager: NotificationManager =
                    context.getSystemService(Context.NOTIFICATION_SERVICE) as
                            NotificationManager
            notificationManager.createNotificationChannel(channel)
        }
    }

    fun showNotification(context: Context,
                         numberOfApps: Int,
                         installAutomatically: Boolean,
                         unmeteredNetworkOnly: Boolean,
                         isConnectedToUnmeteredNetwork: Boolean) {
        with(NotificationManagerCompat.from(context)) {
            createNotificationChannel(context)
            notify(Constants.UPDATES_NOTIFICATION_ID,
                    getNotification(context,
                            numberOfApps,
                            installAutomatically,
                            unmeteredNetworkOnly,
                            isConnectedToUnmeteredNetwork))
        }
    }
}
